@@ -14,6 +14,8 @@
 # limitations under the License.
 #
 import os
+import sys
+
 from typing import Dict, Optional
 
 
@@ -23,8 +25,8 @@ class Operation(object):
     """
 
     def __init__(self, id, type, classifier, filename, runtime_image, memory=None, cpu=None, gpu=None,
-                 dependencies=None, include_subdirectories: bool = False, env_vars=None, inputs=None, outputs=None,
-                 parent_operations=None):
+                 dependencies=None, include_subdirectories: bool = False, env_vars=None, inputs=None,
+                 outputs=None, parent_operations=None):
         """
         :param id: Generated UUID, 128 bit number used as a unique identifier
                    e.g. 123e4567-e89b-12d3-a456-426614174000
@@ -58,6 +60,12 @@ class Operation(object):
             raise ValueError("Invalid pipeline operation: Missing field 'operation filename'.")
         if not runtime_image:
             raise ValueError("Invalid pipeline operation: Missing field 'operation runtime image'.")
+        if cpu and not _validate_range(cpu, min_value=1):
+            raise ValueError("Invalid pipeline operation: CPU must be a positive value or None")
+        if gpu and not _validate_range(gpu, min_value=0):
+            raise ValueError("Invalid pipeline operation: GPU must be a positive value or None")
+        if memory and not _validate_range(memory, min_value=1):
+            raise ValueError("Invalid pipeline operation: Memory must be a positive value or None")
 
         self._id = id
         self._type = type
@@ -123,10 +131,11 @@ class Operation(object):
         return self._gpu
 
     def env_vars_as_dict(self, logger: Optional[object] = None) -> Dict:
-        """Operation stores environment variables in a list of name=value pairs, while
-           subprocess.run() requires a dictionary - so we must convert.  If no envs are
-           configured on the Operation, the existing env is returned, otherwise envs
-           configured on the Operation are overlayed on the existing env.
+        """
+        Operation stores environment variables in a list of name=value pairs, while
+        subprocess.run() requires a dictionary - so we must convert.  If no envs are
+        configured on the Operation, the existing env is returned, otherwise envs
+        configured on the Operation are overlayed on the existing env.
         """
         envs = {}
         for nv in self.env_vars:
@@ -209,7 +218,7 @@ class Pipeline(object):
     Represents a single pipeline constructed in the pipeline editor
     """
 
-    def __init__(self, id, name, runtime, runtime_config):
+    def __init__(self, id, name, runtime, runtime_config, source=None):
         """
         :param id: Generated UUID, 128 bit number used as a unique identifier
                    e.g. 123e4567-e89b-12d3-a456-426614174000
@@ -218,6 +227,7 @@ class Pipeline(object):
         :param runtime: Type of runtime we want to use to execute our pipeline
                         e.g. kfp OR airflow
         :param runtime_config: Runtime configuration that should be used to submit the pipeline to execution
+        :param source: The pipeline source, e.g. a pipeline file or a notebook.
         """
 
         if not name:
@@ -229,6 +239,7 @@ class Pipeline(object):
 
         self._id = id
         self._name = name
+        self._source = source
         self._runtime = runtime
         self._runtime_config = runtime_config
         self._operations = {}
@@ -241,6 +252,10 @@ class Pipeline(object):
     def name(self):
         return self._name
 
+    @property
+    def source(self):
+        return self._source
+
     @property
     def runtime(self):
         """
@@ -263,6 +278,22 @@ class Pipeline(object):
         if isinstance(self, other.__class__):
             return self.id == other.id and \
                 self.name == other.name and \
+                self.source == other.source and \
                 self.runtime == other.runtime and \
                 self.runtime_config == other.runtime_config and \
                 self.operations == other.operations
+
+###########################
+# Utility functions
+###########################
+
+
+def _validate_range(value: str, min_value=0, max_value=sys.maxsize) -> bool:
+    is_valid = False
+
+    if value is None:
+        is_valid = True
+    elif int(value) in range(min_value, max_value):
+        is_valid = True
+
+    return is_valid
